import numpy as np
import pandas as pd
import scipy as sc
import matplotlib.pyplot as plt
import usbtmc
import uuid, os

from .Oscillo import _Oscillo
    

class Keyseight(_Oscillo):
    __version__ = 'Keyseight-1.0.0'
    
    modelDict = {
    }
            
    def __init__(self, device = None, ident = (None, None)):
        super().__init__(device, ident)
        ## Get Number of analog channels on scope
        IDN = self.instr.ask("*IDN?")
        ## Parse IDN
        IDN = IDN.split(',') # IDN parts are separated by commas, so parse on the commas
        MODEL = IDN[1]
        if list(MODEL[1]) == "9": # This is the test for the PXIe scope, M942xA)
            self.NUMBER_ANALOG_CHS = 2
        else:
            self.NUMBER_ANALOG_CHS = int(MODEL[len(MODEL)-2])
        
    def isActiveChannel(self, ch):
        return self.instr.ask(":STATus? {}".format(ch))=="1"
    
    
    def getChannelList(self):
        channelList = ["CHAN{}".format(ch+1) for ch in range(self.NUMBER_ANALOG_CHS)]
        channelList.append("MATH")
        channelList.append("FFT")
        #[channelList.append("MEM{}".format(ch+1)) for ch in range(self.NUMBER_ANALOG_CHS)]
        return channelList
    
    def getActiveChannelList(self):
        activeChannelList = [ ch for ch in self.getChannelList()  if self.isActiveChannel(ch) ]
        return activeChannelList
        
    
    def getChannelData(self, channel='CHAN1'): 
        self.instr.write(":WAVeform:SOURce {}".format(channel))
        # print(self.instr.ask(":WAVeform:POINts?"))
        self.instr.write(":WAVeform:FORMat WORD")
        self.instr.write(":WAVeform:BYTeorder LSBFirst")
        self.instr.write(":WAVeform:UNSigned 0")        
        
        tmp = self.instr.ask(":WAVeform:PREamble?")
        preamble = tmp.split(',')
        (format, type, points, count, xincr, xorg, xref, yincr, yorg, yref) = [eval(st) for st in preamble ]         
        self.instr.write(":WAVeform:DATA?")
        raw = self.instr.read_raw()
        Header = str(raw[0:12])
        startpos = Header.find("#")
        Size_of_Length = int(Header[startpos+1])
        Image_Size = int(Header[startpos+2:startpos+2+Size_of_Length])
        offset = startpos+2+Size_of_Length
        raw = raw[offset:offset+Image_Size-2]
        V = (np.frombuffer(raw, dtype=np.int16)-yref)*yincr+yorg
        t = (np.arange(0,len(V)) - xref) * xincr + xorg
        return (t,V)        
    
    def getHardcopy(self):
        self.instr.write(":SYSTEM:DSP")
        self.instr.write(":HARDCOPY:INKSAVER OFF")
        self.instr.write(":DISPlAY:DATA? PNG, COLOR")
        raw=self.instr.read_raw()
        Header = str(raw[0:12])
        startpos = Header.find("#")
        Size_of_Length = int(Header[startpos+1])
        Image_Size = int(Header[startpos+2:startpos+2+Size_of_Length])
        offset = startpos+Size_of_Length
        raw = raw[offset:offset+Image_Size]        
        filename = "/tmp/{}.bmp".format(uuid.uuid4())
        f = open(filename, "w+b")
        f.write(raw)
        f.close()
        im = plt.imread(filename)
        os.remove(filename)
        return im
       



import matplotlib.pyplot as plt
from Oscillo import Oscillo

devList = Oscillo.getDeviceList()
osc = Oscillo.openDevice(devList[0])

hc = osc.getHardcopy()
plt.imsave('hardcopy.png', hc)

plt.figure()
plt.imshow(hc)
plt.show()

activeChannelList = osc.getActiveChannelList()
data = osc.getChannelListData(activeChannelList)
plt.figure()
osc.plotData(data)
plt.show()



import usbtmc
import matplotlib.pyplot as plt


class Oscillo:
    __version__ = "1.1.0-0"

    def openKeyseightDevice(device, model):
        
        if (serie == 'DSO-X'):
            return DSOX(device)
        raise NameError('Série {} de Keyseight non géré'.format(serie))

    
    def openDevice(device):
        from .Tektronix_TDS import TDS
        from .Tektronix_TBS import TBS
        from .Keyseight_DSOX import DSOX

        (vendor, model) = usbtmc.Instrument(device).ask('*IDN?').split(',')[0:2]
        (serie, model) = model.split(' ')
        if vendor == 'TEKTRONIX':
            if serie == 'TDS':
                return TDS(device)
            if serie == 'TBS':
                return TBS(device)
            raise NameError('Série {} de Keyseight non géré'.format(serie))
        if vendor == 'KEYSIGHT TECHNOLOGIES':
            if (serie == 'DSO-X'):
                return DSOX(device)
            raise NameError('Série {} de Keyseight non géré'.format(serie))
        raise NameError('Vendeur {} non géré'.format(vendor))
        
    def getVersion():
        return Oscillo.__version__

    def getDeviceList():
        devList = usbtmc.list_devices()
        return devList
    
    def getDeviceListId(devList):
        IdList = [usbtmc.Instrument(dev).ask('*IDN?') for dev in devList]
        return IdList
    
    
class _Oscillo:
    __version__ = "0.0.1-0"
    
    colorList = ('orange', 'lightgreen', 'blue', 'lightpink', 'lightcoral')
    rightDec = (0.95, 0.90, 0.75, 0.65, 0.60)
    axesDec =  (0.00, 0.00, 0.20, 0.20, 0.20)
    
    def getVersion(self):
        return self.__version__
    
    def __init__(self, device = None, ident = (None, None), model= None):
        """ Initialisation de l'instrument
        
            utilisation :
            devList = Oscillo.getDeviceList()            
            osc = Oscillo(ident=(0x0699, 0x03aa))
            ou
            osc = Oscillo(devList[0])
            
        """
        if device is None:
            self.instr =  usbtmc.Instrument(ident[0], ident[1]) 
        else:
            self.instr = usbtmc.Instrument(device)            
        
        
    def getId(self):
        return self.instr.ask('*IDN?')
            
    def getChannelList(self):
        raise NameError("getChannelList n'est pas encore implémenté pour cet oscilloscope")

    def getActiveChannelList(self):
        raise NameError("getActiveChannelList n'est pas encore implémenté pour cet oscilloscope")
        
    def getHardcopy(self):
        raise NameError("getHardcopy n'est pas encore implémenté pour cet oscilloscope")
        
    def getChannelData(self):
        raise NameError("getChannelData n'est pas encore implémenté pour cet oscilloscope")
        
    def getChannelListData(self, channelList):
        res = {}
        for channel in channelList:
            (t, V) = self.getChannelData(channel)        
            if 't' not in res:
                res['t'] = t
            res[channel]= V
        return res
            
    
    def plotData(self, data):
        axesIndex = len(data)-1;
        fig, ax = plt.subplots(1,1)
        fig.subplots_adjust(right=self.rightDec[axesIndex])
        count = 0;
        for ch in data:
            if ch == 't':
                t = data['t']
            else:
                if count > 0:
                    tax = ax.twinx()
                else:
                    tax = ax 

                tax.plot(t, data[ch], color=self.colorList[count], linestyle='solid')
                tax.set_xlabel('time (s)')
                tax.set_ylabel(ch, color=self.colorList[count])
                tax.tick_params(axis='y', colors=self.colorList[count])
                if count>0:
                    tax.spines['right'].set_position(('axes', 1.0+(count-1)*self.axesDec[axesIndex]))
                count += 1

                tax.set_frame_on(True)
                tax.patch.set_visible(False)
        



from .Keyseight import Keyseight


class DSOX(Keyseight):
    __version__ = 'DSOX-1.0.0'

    modelDict = {
        "1102A": (0x2a8d, 0x1787),
    }

    def __init__(self, device = None, ident = (None, None), model= None):
        if model is None:
            super().__init__(device, ident)
            return
        if model in self.modelDict:
            super().__init__(ident = self.modelDict[model])
            return
        
        print('Liste des modèmles de '+self.__class__.__name__+' référencés')
        print(self.modelDict)
        raise NameError("le modèle {} n'est pour le moment pas référencé".format(model))
        



"""
Linfitxy V2.0
Dev : Marc-Antoine Verdier sur la base de la version Matlab par Julien Browaeys
et Tristan Beau
Pour tout bug ou remarque : marc-antoine.verdier@univ-paris-diderot.fr
"""
import numpy as np
import numpy.random as rd
import matplotlib.pyplot as plt
import scipy.optimize as opt
import scipy.special as spe


def __fun(p, x, y, sx, sy):
        return (y - p[0]*x - p[1]) / np.sqrt((p[0]*sx)**2 + sy**2)

def __lineq(x, a, b):
    return a*x + b


def __val2str(nsig, val):
    if nsig < 1:
        pre = -int(nsig - 2)
        return "{:.{}f}".format(val, pre)
    else:
        pre = 10**(int(nsig))
        return "{:.0f}".format(np.round(val / pre) * pre)


def __make_plot(x, y, dx, dy, p, s, re=None, l_ind=None, h_ind=None,
                marker=None, markercolor=None, linecolor=None, conf_line=None):
    nhull = 100
    if dx is not None:
        xplot = np.linspace(x[0]-dx[0], x[-1]+dx[-1], nhull)
    else:
        xplot = np.linspace(x[0]-(x[1]-x[0])/2, x[-1]+(x[-1]-x[-2])/2, nhull)
    yplot = xplot * p[0] + p[1]
    plt.figure()
    plt.errorbar(x, y, xerr=dx, yerr=dy, fmt=marker, c=markercolor,
                 ecolor=markercolor)
    plt.plot(xplot, yplot,  color=linecolor)
    nd0 = np.log10(s[0])
    nd1 = np.log10(s[1])
    fmt_p0 = __val2str(nd0, p[0])
    fmt_s0 = __val2str(nd0, s[0])
    fmt_p1 = __val2str(nd1, p[1])
    fmt_s1 = __val2str(nd1, s[1])
    plt.title(r'Fit: y = (' + fmt_p0 + r' $\pm$ ' + fmt_s0
              + r') x + (' + fmt_p1 + r' $\pm$ ' + fmt_s1 + r')')
    if conf_line:
        axipbi = xplot * np.ones((re.shape[0], xplot.shape[0])) * \
            np.meshgrid(np.ones(xplot.shape[0]), re[:, 0])[1] + \
            np.meshgrid(np.ones(xplot.shape[0]), re[:, 1])[1]
        saxipbi = np.sort(axipbi, axis=0)
        plt.plot(xplot, saxipbi[l_ind], '--', color=linecolor)
        plt.plot(xplot, saxipbi[h_ind], '--', color=linecolor)

def __fit_xerr_yerr(x, y, dx, dy, nbloop, nsigma, plot, conf_line, marker,
                    markercolor, linecolor, **kwargs):
    re = np.zeros((nbloop, 2))
    a0 = np.mean((y[1:]-y[:-1]) / (x[1:]-x[:-1]))
    b0 = np.mean(y - (a0 * x))
    p0 = np.array([a0, b0])
    lst = opt.least_squares(__fun, p0, args=(x, y, dx, dy), **kwargs)
    popt = lst.x
    for i in range(nbloop):
        xi = rd.normal(x, dx)
        yi = rd.normal(y, dy)
        lst = opt.least_squares(__fun, popt, args=(xi, yi, dx, dy))
        popt = lst.x
        re[i] = popt
    thres = 0.5 * spe.erfc(nsigma / np.sqrt(2))
    p_low = np.zeros(re.shape[1])
    p_hig = np.zeros(re.shape[1])
    l_ind = int(1 + np.floor(nbloop * thres))
    h_ind = int(np.ceil(nbloop * (1 - thres)))
    for i in range(re.shape[1]):
        sorted_pt = np.sort(re[:, i])
        p_low[i] = sorted_pt[l_ind]
        p_hig[i] = sorted_pt[h_ind]
    p = (p_low + p_hig) / 2
    s = p_hig - p
    if plot:
        __make_plot(x, y, dx, dy, p, s, re, l_ind, h_ind, marker, markercolor,
                    linecolor, conf_line)
    return np.array([p[0], p[1], s[0], s[1]])


def __fit(x, y, dy, plot, marker, markercolor, linecolor, **kwargs):
    a0 = np.mean((y[1:]-y[:-1]) / (x[1:]-x[:-1]))
    b0 = np.mean(y - (a0 * x))
    p0 = np.array([a0, b0])
    popt, pcov = opt.curve_fit(__lineq, x, y, p0=p0, sigma=dy,
                               absolute_sigma=True)
    perr = np.sqrt(np.diag(pcov))
    if plot:
        __make_plot(x, y, None, dy, popt, perr, None, None, None, marker,
                    markercolor, linecolor, False)
    return np.array([popt[0], popt[1], perr[0], perr[1]])


def linfitxy(x, y, dx=None, dy=None, nbloop=500, nsigma=1, plot=True,
             conf_line=True, marker='o', markercolor='tab:blue',
             linecolor='tab:orange', **kwargs):
    if (dx is not None) and (dy is not None):
        return __fit_xerr_yerr(x, y, dx, dy, nbloop, nsigma, plot, conf_line,
                               marker, markercolor, linecolor, **kwargs)
    else:
        return __fit(x, y, dy, plot, marker, markercolor, linecolor, **kwargs)



#!/usr/bin/env python
# coding: utf-8
from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *

import sys, uuid, time, getopt
import traceback

from matplotlib import pyplot as plt
from Oscillo import Oscillo
import numpy, scipy.io, pandas

forHelp = False
forHardcopy = False
forData = False
forGraph = False
fromBash = False

dataTransfered = False
splash = True
win = None
app = None
hc = None
outputDir = '/tmp'
devList = None
devNum = 0

def doHelp():
  print('TDS.py -hHDGN')
  print('-h, --help        print this help')
  print('-H, --hardcopy    get hardcopy of the scope')
  print('-D, --data        get data of each channel of the scope')
  print('-b, --bash       when used from bash script')
  print('-G, --graph       reproduce a graph as the scope screen')
  print('-d dir, --dir=dir specifies the directory where files are written')
  print('-N, --nosplash    dont open a splash screen')


class OscSignals(QObject):
  finished = pyqtSignal()
  error = pyqtSignal(tuple)
  result = pyqtSignal(object)
  progress = pyqtSignal(int)


class Osc(QRunnable):
  def __init__(self, fn, *args, **kwargs):
    super(Osc, self).__init__()

    # Store constructor arguments (re-used for processing)
    self.fn = fn
    self.args = args
    self.kwargs = kwargs
    self.signals = OscSignals()    

    # Add the callback to our kwargs
    self.kwargs['progress_callback'] = self.signals.progress

  @pyqtSlot()
  def run(self):        
    # Retrieve args/kwargs here; and fire processing using them
    try:
      result = self.fn(*self.args, **self.kwargs)
    except:
      traceback.print_exc()
      exctype, value = sys.exc_info()[:2]
      self.signals.error.emit((exctype, value, traceback.format_exc()))
    else:
      self.signals.result.emit(result)  # Return the result of the processing
    finally:
      self.signals.finished.emit()  # Done
        
class MainWindow(QMainWindow):
  def __init__(self, *args, **kwargs):
    super(MainWindow, self).__init__(*args, **kwargs)

    self.devNum = devNum
    
    if not splash:
      self.doStart()
      return

    layout = QVBoxLayout()
    
    self.pbar = QProgressBar(self)
    self.pbar.setGeometry(0, 0, 300, 25)
    self.pbar.setMaximum(0)
    
    self.l = QLabel(self)
    # self.l.setText(type(self.osc).__name__)
    self.l.setText("Prêt ?")
    self.l.move(0,30)

    self.cb = QComboBox()
    self.cb.addItems(["{} - {}".format(dev.manufacturer, dev.product) for dev in devList])
    self.cb.setCurrentIndex(self.devNum)
    self.cb.currentIndexChanged.connect(self.selectionChange)

    self.forGraph = QCheckBox("Graphique")
    self.forGraph.setChecked(forGraph)
    # self.forGraph.stateChanged.connect(lambda:self.btnstate(self.forGraph))
    layout.addWidget(self.forGraph)
  
    self.forHardcopy = QCheckBox("Hardcopy")
    self.forHardcopy.setChecked(forHardcopy)
    # self.forHardcopy.stateChanged.connect(lambda:self.btnstate(self.forHardcopy))
    layout.addWidget(self.forHardcopy)

    self.forData = QCheckBox("Data")
    self.forData.setChecked(forData)
    # self.forData.stateChanged.connect(lambda:self.btnstate(self.forData))
    layout.addWidget(self.forData)


    self.b = QPushButton("Commencer !")
    self.b.pressed.connect(self.doStart)

    layout.addWidget(self.cb)
    layout.addWidget(self.l)    
    layout.addWidget(self.pbar)
    self.pbar.setVisible(False)
    layout.addWidget(self.b)

    w = QWidget()
    w.setLayout(layout)

    self.setCentralWidget(w)
    self.show()
    self.threadpool = QThreadPool()
  
    self.timer = QTimer()
    self.timer.setInterval(1000)
    self.timer.timeout.connect(self.recurring_timer)
    self.timer.start()
    return

  def selectionChange(self, i):
    self.devNum = i
    return

  def progress_fn(self, n):
    return

#retrieve data from the oscilloscope
  def getData(self):
    global dataTransfered, data
    if dataTransfered:
      return
    if splash:
      self.l.setText("Transfert des données")
    activeChannelList = self.osc.getActiveChannelList()
    data = self.osc.getChannelListData(activeChannelList)
    dataTransfered = True

  def doGraph(self):
    self.getData()
    return

  def doHardcopy(self):
    global hc
    if outputDir == "/tmp":
      fileName = outputDir+'/'+str(uuid.uuid1())+".png"
    else:
      fileName = outputDir+"/hardcopy.png"
    print(fileName)
    
    if splash:
      self.l.setText("Transfert de hardcopy")
    hc = self.osc.getHardcopy()
    plt.imsave(fileName, hc)
    if splash:
      self.l.setText("doHardcopy Fait !")
    return

  def doData(self):
    if outputDir == "/tmp":
      fileName = outputDir+'/'+str(uuid.uuid1())+".txt"
    else:
      fileName = outputDir+"/data.txt"
  
    self.getData()
    if splash:
      self.l.setText("Writing csv File")
    df = pandas.DataFrame(data)
    df.to_csv(fileName)
        
    if splash:
      self.l.setText("Writing mat File")
    scipy.io.savemat( outputDir+'/data.mat', mdict={'data': data})    
    print(fileName)
    sys.stdout.flush()
    if splash:
      self.l.setText("doData Fait !")
    return

  def doTruc(self):
    if splash:
      self.pbar.setVisible(True)
    if forGraph:
        self.doGraph()
    if forHardcopy:
        self.doHardcopy()
    if forData:
        self.doData()
    if forGraph:
        self.doGraph()
    if splash:
      self.pbar.setVisible(False)     
    return

  def execute_this_fn(self, progress_callback):
    global forGraph, forData, forHardcopy

    forGraph = self.forGraph.isChecked()
    forHardcopy = self.forHardcopy.isChecked()
    forData = self.forData.isChecked()

    self.doTruc()

    return "Done."

  def print_output(self, s):
    return

  def thread_complete(self):
    self.close()
    return

  def doStart(self):
    # Pass the function to execute
    if splash:
      self.pbar.setVisible(True)
      self.b.setVisible(False)

    self.osc = Oscillo.openDevice(devList[self.devNum])

    if not splash:
      self.doTruc()
      return

    osc = Osc(self.execute_this_fn) # Any other args, kwargs are passed to the run function
    osc.signals.result.connect(self.print_output)
    osc.signals.finished.connect(self.thread_complete)
    osc.signals.progress.connect(self.progress_fn)
    
    # Execute
    self.threadpool.start(osc)
    
      
  def recurring_timer(self):
    return

def doFromBash():
  global fromBash
  
  fromBash = True


def doNoSplash():
  global splash
  splash = False

def startSplash():
  global splash, app, win
  app = QApplication([])
  win = MainWindow()
  return

def endSplash():
  return

def init():
  if splash:
    app.exec_()    
  return


def main(argv):
  global outputDir, forHelp, forHardcopy, forData, forGraph, devList, devNum, hc
  
  try:
    opts, args = getopt.getopt(argv,"hbHDGNd:", ["help", "bash", "hardcopy","data", "graph","nosplash","dir=","dev="])
  except getopt.GetoptError:
    doHelp()
    sys.exit(2)

  for opt, arg in opts:
    if opt in ('-h','--help'):
      forHelp = True
    elif opt in ('-b','--bash'):
      doFromBash()
    elif opt in ('-N','--nosplash'):
      doNoSplash()
    elif opt in ('-H','--hardcopy'):          
      forHardcopy = True
    elif opt in ("-D", "--data"):
      forData = True
    elif opt in ("-G", "--graph"):
      forGraph = True
    elif opt in ("-d", "--dir"):
      outputDir = arg
    elif opt in ("--dev"):
      devNum = int(arg)
 
  if forHelp:
    doHelp()
    exit(0)

  devList = Oscillo.getDeviceList()
  startSplash()

  init()
  
  if not fromBash:
    if forHardcopy:
      fig, ax = plt.subplots()
      plt.imshow(hc)
      ax.axis('off')

    if forGraph:
      win.osc.plotData(data)

    if forHardcopy or forGraph:  
      plt.show() 

  endSplash()

main(sys.argv[1:])



import setuptools

with open("README.md", "r") as fh:
	long_description = fh.read()

setuptools.setup(
	name="tpp7", # Replace with your own username
	version="1.0.16",
	author="Olivier Cardoso",
	author_email="Olivier.Cardoso@univ-paris-diderot.fr",
	description="Un paquetage pour les TP de Physique de l'université Paris-Diderot",
	long_description=long_description,
	long_description_content_type="text/markdown",
	url="https://github.com/cardosoo/tp",
	package_dir={'': 'src'},
	packages=setuptools.find_packages(where='src'),
	scripts=['tests/Osc.py', 'tests/Osc2.py', 'tests/minimalTPUsage.py'],
	data_files=[('images', ['tests/images/disk.png', 'tests/images/disk-pencil.png'])],
	classifiers=[
		"Programming Language :: Python :: 3",
		"License :: OSI Approved :: MIT License",
		"Operating System :: OS Independent",
	],
	install_requires=[
		'numpy', 'scipy', 'matplotlib', 'python-usbtmc', 'pandas', 'pyusb'
	],
	python_requires='>=3.6',
)



import numpy as np
import pandas as pd
import scipy as sc
import matplotlib.pyplot as plt
import usbtmc
import uuid, os

from .Oscillo import _Oscillo
        
class Tektronix(_Oscillo):
    __version__ = 'Tektronix-1.0.0'
    
    modelDict = {
    }
            
    def __init__(self, device = None, ident = (None, None)):
        super().__init__(device, ident)
    
    
    def getChannelList(self):
        self.instr.write('HEAD ON')
        rep = self.instr.ask('SEL?').split(':')[2]
        channelList = [st.split(' ')[0] for st in rep.split(';')]
        return channelList
    
    def getActiveChannelList(self):
        self.instr.write('HEAD ON')
        rep = self.instr.ask('SEL?').split(':')[2]
        channelList = [st.split(' ')[0] for st in rep.split(';') if st.split(' ')[1] is '1']
        return channelList
        
    
    def getChannelData(self, channel='CH1'): 
        self.instr.write("HEADER OFF")
        self.instr.write("dat:sou "+channel)
        self.instr.write("DATa:ENCdg SRibinary")
        self.instr.write("DATa:WIDth 2")
        self.instr.write("WFMP:ENC BIN")
        preamble =  self.instr.ask("WFMPre?").split(';')
        (xincr, ptoff, xzero, xunit, ymult, yzero, yoff, yunit) = [eval(st) for st in preamble[8:16]]
        self.instr.write("curve?")
        curve = self.instr.read_raw()
        V = yzero+ymult*(np.frombuffer(curve[6:-1], dtype=np.int16)-yoff)
        
        # V = yzero+ymult*(np.array([int(st) for st in curve])-yoff)
        
        #self.instr.write("WFMP:ENC ASC")
        #curve = self.instr.ask("curve?")
        #V = yzero+ymult*(np.array([int(st) for st in curve.split(',')])-yoff)
        t = xzero+xincr*(np.arange(0,len(V))-ptoff)
        #return (t[6:-1],V[6:-1])
        return (t,V)
    
    def getHardcopy(self):
        self.instr.write("HARDC:PORT USB")
        self.instr.write("HARDC:BUTTON SAVES")
        self.instr.write("HARDC:LAY PORTR")
        self.instr.write("HARDC:FORM JPG")
        self.instr.write("HARDC START")
        raw=self.instr.read_raw()
        filename = "/tmp/{}.bmp".format(uuid.uuid4())
        f = open(filename, "w+b")
        f.write(raw)
        f.close()
        im = plt.imread(filename)
        os.remove(filename)
        return im



#!/usr/bin/env python
# coding: utf-8
# https://www.learnpyqt.com/examples/no2pads-simple-notepad-clone/


import os

from PyQt5.QtGui import *
from PyQt5.QtWidgets import *
from PyQt5.QtCore import *

import sys, uuid, time, getopt
import traceback

import matplotlib
from matplotlib import pyplot as plt
from Oscillo import Oscillo
import numpy, scipy.io, pandas

forHelp = False
forHardcopy = False
forData = False
forGraph = False
fromBash = False

dataTransfered = False
splash = True
win = None
winData = None
app = None
hc = None
outputDir = '/tmp'
devList = None
devNum = 0


matplotlib.use('Qt5Agg')

def doHelp():
  print('TDS.py -hHDGN')
  print('-h, --help        print this help')
  print('-H, --hardcopy    get hardcopy of the scope')
  print('-D, --data        get data of each channel of the scope')
  print('-b, --bash       when used from bash script')
  print('-G, --graph       reproduce a graph as the scope screen')
  print('-d dir, --dir=dir specifies the directory where files are written')
  print('-N, --nosplash    dont open a splash screen')


class OscSignals(QObject):
  finished = pyqtSignal()
  error = pyqtSignal(tuple)
  result = pyqtSignal(object)
  progress = pyqtSignal(int)


class Osc(QRunnable):
  def __init__(self, fn, *args, **kwargs):
    super(Osc, self).__init__()

    # Store constructor arguments (re-used for processing)
    self.fn = fn
    self.args = args
    self.kwargs = kwargs
    self.signals = OscSignals()    

    # Add the callback to our kwargs
    self.kwargs['progress_callback'] = self.signals.progress

  @pyqtSlot()
  def run(self):        
    # Retrieve args/kwargs here; and fire processing using them
    try:
      result = self.fn(*self.args, **self.kwargs)
    except:
      traceback.print_exc()
      exctype, value = sys.exc_info()[:2]
      self.signals.error.emit((exctype, value, traceback.format_exc()))
    else:
      self.signals.result.emit(result)  # Return the result of the processing
    finally:
      self.signals.finished.emit()  # Done

class DataWindow(QMainWindow):
  def __init__(self, dataStr, *args, **kwargs):
    super(QMainWindow, self).__init__(*args, **kwargs)
    self.buildLayout()

    w = QWidget()
    w.setLayout(self.layout)
    self.setCentralWidget(w)
    self.show()

  def buildLayout(self):
    self.file_path = None

    self.save_current_file_shortcut = QShortcut(QKeySequence('Ctrl+S'), self)
    self.save_current_file_shortcut.activated.connect(self.save_current_file)

    self.layout = QVBoxLayout()
    text = "Untitled File"
    self.title = QLabel(text)
    self.title.setWordWrap(True)
    self.title.setAlignment(Qt.AlignCenter)
    self.layout.addWidget(self.title)
  
    self.scrollable_text_area = QTextEdit()
    self.setTexte("Pour le moment il n'y a rien....", show=False)
    self.layout.addWidget(self.scrollable_text_area)

    file_toolbar = QToolBar("File")
    file_toolbar.setIconSize(QSize(14, 14))
    self.addToolBar(file_toolbar)
    file_menu = self.menuBar().addMenu("&File")

    save_file_action = QAction(QIcon(os.path.join('images', 'disk.png')), "Save", self)
    save_file_action.setStatusTip("Save current page")
    save_file_action.triggered.connect(self.file_save)
    file_menu.addAction(save_file_action)
    file_toolbar.addAction(save_file_action)

    saveas_file_action = QAction(QIcon(os.path.join('images', 'disk--pencil.png')), "Save As...", self)
    saveas_file_action.setStatusTip("Save current page to specified file")
    saveas_file_action.triggered.connect(self.file_saveas)
    file_menu.addAction(saveas_file_action)
    file_toolbar.addAction(saveas_file_action)

  def file_save(self):
    if self.path is None:
      # If we do not have a path, we need to use Save As.
      return self.file_saveas()

    self._save_to_path(self.path)

  def file_saveas(self):
    path, _ = QFileDialog.getSaveFileName(self, "Save file", "", "Text documents (*.txt);All files (*.*)")

    if not path:
      # If dialog is cancelled, will return ''
      return

    self._save_to_path(path)

  def _save_to_path(self, path):
    text = self.editor.toPlainText()
    try:
      with open(path, 'w') as f:
        f.write(text)

    except Exception as e:
      self.dialog_critical(str(e))

    else:
      self.path = path
      self.update_title()




  def setTexte(self, dataStr, show=True):
     self.scrollable_text_area.setText(dataStr)
     if show:
      self.show()

  def updateTexte(self, dataStr):
    self.scrollable_text_area.setText(dataStr)
    self.show()

  def save_current_file(self):
    if not self.file_path:
      new_file_path, filter_type = QFileDialog.getSaveFileName(
        self, 
        "Save this file as...",
        "data.csv", 
        "CSV Files (*.csv, *.txt, *.*)")
      if new_file_path:
        self.file_path = new_file_path
      else:
        return False
    file_contents = self.scrollable_text_area.toPlainText()
    with open(self.file_path, "w") as f:
      f.write(file_contents)
    self.title.setText(self.file_path)
  

class MainWindow(QMainWindow):
  textSaved = pyqtSignal(str)
  def __init__(self, *args, **kwargs):
    global winData
    super(MainWindow, self).__init__(*args, **kwargs)

    self.devNum = devNum
    
    if not splash:
      self.doStart()
      return

    w = QWidget()
    self.buildLayout()
    w.setLayout(self.layout)

    self.setCentralWidget(w)
    self.show()

    self.threadpool = QThreadPool()
  
    self.timer = QTimer()
    self.timer.setInterval(1000)
    self.timer.timeout.connect(self.recurring_timer)
    self.timer.start()


    winData = DataWindow("Elle n'y est pas pour rien celle là !")
    self.textSaved.connect(winData.updateTexte)
    return

  def buildLayout(self):
    self.layout = QVBoxLayout()

    self.pbar = QProgressBar(self)
    self.pbar.setGeometry(0, 0, 300, 25)
    self.pbar.setMaximum(0)
    
    self.l = QLabel(self)
    self.l.setText("Prêt ?")
    self.l.move(0,30)

    self.cb = QComboBox()
    self.cb.addItems(["{} - {}".format(dev.manufacturer, dev.product) for dev in devList])
    self.cb.setCurrentIndex(self.devNum)
    self.cb.currentIndexChanged.connect(self.selectionChange)

    self.forGraph = QCheckBox("Graphique")
    self.forGraph.setChecked(forGraph)
    self.layout.addWidget(self.forGraph)
  
    self.forHardcopy = QCheckBox("Hardcopy")
    self.forHardcopy.setChecked(forHardcopy)
    self.layout.addWidget(self.forHardcopy)

    self.forData = QCheckBox("Data")
    self.forData.setChecked(forData)
    self.layout.addWidget(self.forData)


    self.b = QPushButton("Commencer !")
    self.b.pressed.connect(self.doStart)

    self.layout.addWidget(self.cb)
    self.layout.addWidget(self.l)    
    self.layout.addWidget(self.pbar)
    self.pbar.setVisible(False)
    self.layout.addWidget(self.b)

  def pousseSetTexte(self, texte):
    self.textSaved.emit(texte)
    return

  def selectionChange(self, i):
    self.devNum = i
    return

  def progress_fn(self, n):
    return

  def send_signal(self):
    global winData
    if self.counter == 0:
      signal=self.__line.displayText()
      self.Window2=Window2(signal, self)
      self.Window2.show()
      self.textSaved.connect(self.Window2.showMessage)
      self.counter = 1
    else:
      signal = self.__line.displayText()
      self.textSaved.emit(signal)

#retrieve data from the oscilloscope
  def getData(self):
    global dataTransfered, data
    if dataTransfered:
      return
    if splash:
      self.l.setText("Transfert des données")
    activeChannelList = self.osc.getActiveChannelList()
    data = self.osc.getChannelListData(activeChannelList)
    dataTransfered = True

  def doGraph(self):
    self.getData()
    return

  def doHardcopy(self):
    global hc
    if outputDir == "/tmp":
      fileName = outputDir+'/'+str(uuid.uuid1())+".png"
    else:
      fileName = outputDir+"/hardcopy.png"
    print(fileName)
    
    if splash:
      self.l.setText("Transfert de hardcopy")
    hc = self.osc.getHardcopy()
    plt.imsave(fileName, hc)
    if splash:
      self.l.setText("doHardcopy Fait !")
    return

  def doData(self):
    global winData

    if outputDir == "/tmp":
      fileName = outputDir+'/'+str(uuid.uuid1())+".txt"
    else:
      fileName = outputDir+"/data.txt"
  
    self.getData()
    if splash:
      self.l.setText("Writing csv File")
    df = pandas.DataFrame(data)
    df.to_csv(fileName)
    
    # winData = DataWindow(df.to_csv())
    print("Debug 235- avant setText")
    # winData.scrollable_text_area.setText(df.to_csv())
    print("Debug 237- avant show")
    self.pousseSetTexte(df.to_csv())
    winData.show()

    if splash:
      self.l.setText("Writing mat File")
    scipy.io.savemat( outputDir+'/data.mat', mdict={'data': data})    
    print(fileName)
    sys.stdout.flush()
    if splash:
      self.l.setText("doData Fait !")
    return

  def doTruc(self):
    if splash:
      self.pbar.setVisible(True)
    if forData:
        self.doData()
    if forGraph:
        self.doGraph()
    if forHardcopy:
        self.doHardcopy()
    if splash:
      self.pbar.setVisible(False)     
    return

  def execute_this_fn(self, progress_callback):
    global forGraph, forData, forHardcopy

    forGraph = self.forGraph.isChecked()
    forHardcopy = self.forHardcopy.isChecked()
    forData = self.forData.isChecked()
    self.doTruc()
    return "Done."

  def print_output(self, s):
    return

  def thread_complete(self):
    global outputDir, forHelp, forHardcopy, forData, forGraph, devList, devNum, hc
    if not fromBash:
      doNotFromBash()
      self.close() ####################### Si on veut fermer la fenêtre de contrôle une fois que tout est fait...
    return


  def doStart(self):
    # Pass the function to execute
    if splash:
      self.pbar.setVisible(True)
      self.b.setVisible(False)

    self.osc = Oscillo.openDevice(devList[self.devNum])

    if not splash:
      self.doTruc()
      return

    osc = Osc(self.execute_this_fn) # Any other args, kwargs are passed to the run function
    osc.signals.result.connect(self.print_output)
    osc.signals.finished.connect(self.thread_complete)
    osc.signals.progress.connect(self.progress_fn)
    
    # Execute
    self.threadpool.start(osc)
    
      
  def recurring_timer(self):
    return

def doNotFromBash():
  global outputDir, forHelp, forHardcopy, forData, forGraph, devList, devNum, hc
  if forHardcopy:
    fig, ax = plt.subplots()
    plt.imshow(hc)
    ax.axis('off')

  if forGraph:
    win.osc.plotData(data)

  if forHardcopy or forGraph:  
    plt.show() 


def doFromBash():
  global fromBash
  fromBash = True


def doNoSplash():
  global splash
  splash = False

def startSplash():
  global splash, app, win, winData
  app = QApplication([])
  win = MainWindow()
  winData.hide()
  return

def endSplash():
  return

def init():
  if splash:
    print("avant app.exec_()")
    app.exec_()    
    print("après app.exec_()")
  return


def main(argv):
  global outputDir, forHelp, forHardcopy, forData, forGraph, devList, devNum, hc
  
  try:
    opts, args = getopt.getopt(argv,"hbHDGNd:", ["help", "bash", "hardcopy","data", "graph","nosplash","dir=","dev="])
  except getopt.GetoptError:
    doHelp()
    sys.exit(2)

  for opt, arg in opts:
    if opt in ('-h','--help'):
      forHelp = True
    elif opt in ('-b','--bash'):
      doFromBash()
    elif opt in ('-N','--nosplash'):
      doNoSplash()
    elif opt in ('-H','--hardcopy'):          
      forHardcopy = True
    elif opt in ("-D", "--data"):
      forData = True
    elif opt in ("-G", "--graph"):
      forGraph = True
    elif opt in ("-d", "--dir"):
      outputDir = arg
    elif opt in ("--dev"):
      devNum = int(arg)
 
  if forHelp:
    doHelp()
    exit(0)

  devList = Oscillo.getDeviceList()
  startSplash()
  print("Après le startSplash...")
  init()
  print("Après le init...")
  
  if not fromBash:
    print("debug 393- avant notFromBash")
    # doNotFromBash() #### Il faut revoir cette organisation qui n'est pas bonne...
    print("debug 395- après notFromBash")

  endSplash()


  print("Là on est sorti en fait...")
  # winData = DataWindow("Peut-être que celle-là est bonne ?")
main(sys.argv[1:])
